@@ -1,3 +1,92 @@
+/***************************************************************************
+ * ncat_exec_win.c -- Windows-specific subprocess execution.               *
+ ***********************IMPORTANT NMAP LICENSE TERMS************************
+ *                                                                         *
+ * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
+ *                                                                         *
+ * Note that the GPL places important restrictions on "derived works", yet *
+ * it does not provide a detailed definition of that term.  To avoid       *
+ * misunderstandings, we consider an application to constitute a           *
+ * "derivative work" for the purpose of this license if it does any of the *
+ * following:                                                              *
+ * o Integrates source code from Nmap                                      *
+ * o Reads or includes Nmap copyrighted data files, such as                *
+ *   nmap-os-db or nmap-service-probes.                                    *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
+ * o Links to a library or executes a program that does any of the above   *
+ *                                                                         *
+ * The term "Nmap" should be taken to also include any portions or derived *
+ * works of Nmap.  This list is not exclusive, but is meant to clarify our *
+ * interpretation of derived works with some common examples.  Our         *
+ * interpretation applies only to Nmap--we don't speak for other people's  *
+ * GPL works.                                                              *
+ *                                                                         *
+ * If you have any questions about the GPL licensing restrictions on using *
+ * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
+ * we also offer alternative license to integrate Nmap into proprietary    *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
+ * sales@insecure.com for further information.                             *
+ *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
+ * If you received these files with a written license agreement or         *
+ * contract stating terms other than the terms above, then that            *
+ * alternative license agreement takes precedence over these comments.     *
+ *                                                                         *
+ * Source is provided to this software because we believe users have a     *
+ * right to know exactly what a program is going to do before they run it. *
+ * This also allows you to audit the software for security holes (none     *
+ * have been found so far).                                                *
+ *                                                                         *
+ * Source code also allows you to port Nmap to new platforms, fix bugs,    *
+ * and add new features.  You are highly encouraged to send your changes   *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
+ * distribution.  By sending these changes to Fyodor or one of the         *
+ * Insecure.Org development mailing lists, it is assumed that you are      *
+ * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because the *
+ * inability to relicense code has caused devastating problems for other   *
+ * Free Software projects (such as KDE and NASM).  We also occasionally    *
+ * relicense the code to third parties as discussed above.  If you wish to *
+ * specify special license conditions of your contributions, just say so   *
+ * when you send them.                                                     *
+ *                                                                         *
+ * This program is distributed in the hope that it will be useful, but     *
+ * WITHOUT ANY WARRANTY; without even the implied warranty of              *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
+ *                                                                         *
+ ***************************************************************************/
+
+/* $Id$ */
+
 #include <assert.h>
 
 #include "ncat.h"
@@ -21,11 +110,18 @@ static int atexit_registered = 0;
 static void terminate_subprocesses(void);
 static void sigint_handler(int s);
 
+/* This may be set with set_pseudo_sigchld_handler. It is called when a thread
+   representing a child process ends. */
+static void (*pseudo_sigchld_handler)(void) = NULL;
+/* Simulates blocking of SIGCHLD while the handler runs. Also prevents
+   concurrent modification of pseudo_sigchld_handler. */
+static HANDLE pseudo_sigchld_mutex = NULL;
+
 /* This structure holds information about a subprocess with redirected input
    and output handles. */
 struct subprocess_info {
     HANDLE proc;
-    SOCKET fd;
+    struct fdinfo fdn;
     HANDLE child_in_r;
     HANDLE child_in_w;
     HANDLE child_out_r;
@@ -34,18 +130,19 @@ struct subprocess_info {
 
 /* Run a child process, redirecting its standard file handles to a socket
    descriptor. Return the child's PID or -1 on error. */
-int netrun(int fd, char *cmdexec)
+int netrun(struct fdinfo *fdn, char *cmdexec)
 {
     struct subprocess_info *info;
     HANDLE thread;
     int pid;
 
     info = (struct subprocess_info *) safe_malloc(sizeof(*info));
-    info->fd = fd;
+    info->fdn = *fdn;
 
     pid = start_subprocess(cmdexec, info);
     if (pid == -1) {
         free(info);
+        close(info->fdn.fd);
         return -1;
     }
 
@@ -63,14 +160,14 @@ int netrun(int fd, char *cmdexec)
 }
 
 /* Run the given command line as if by exec. Doesn't return. */
-void netexec(int fd, char *cmdexec)
+void netexec(struct fdinfo *fdn, char *cmdexec)
 {
     struct subprocess_info *info;
     int pid;
     DWORD ret;
 
     info = (struct subprocess_info *) safe_malloc(sizeof(*info));
-    info->fd = fd;
+    info->fdn = *fdn;
 
     pid = start_subprocess(cmdexec, info);
     if (pid == -1)
@@ -81,6 +178,19 @@ void netexec(int fd, char *cmdexec)
     ExitProcess(subprocess_thread_func(info));
 }
 
+/* Set a pseudo-signal handler that is called when a thread representing a
+   child process dies. This is only used on Windows. */
+extern void set_pseudo_sigchld_handler(void (*handler)(void))
+{
+    if (pseudo_sigchld_mutex == NULL) {
+        pseudo_sigchld_mutex = CreateMutex(NULL, FALSE, NULL);
+        assert(pseudo_sigchld_mutex != NULL);
+    }
+    assert(WaitForSingleObject(pseudo_sigchld_mutex, INFINITE) == WAIT_OBJECT_0);
+    pseudo_sigchld_handler = handler;
+    assert(ReleaseMutex(pseudo_sigchld_mutex) != 0);
+}
+
 /* Run a command and redirect its input and output handles to a pair of
    anonymous pipes.  The process handle and pipe handles are returned in the
    info struct. Returns the PID of the new process, or -1 on error. */
@@ -173,7 +283,13 @@ static const char *get_shell(void)
 
 static void subprocess_info_close(struct subprocess_info *info)
 {
-    closesocket(info->fd);
+#ifdef HAVE_OPENSSL
+    if (info->fdn.ssl != NULL) {
+        SSL_shutdown(info->fdn.ssl);
+        SSL_free(info->fdn.ssl);
+    }
+#endif
+    closesocket(info->fdn.fd);
     CloseHandle(info->proc);
     CloseHandle(info->child_in_r);
     CloseHandle(info->child_in_w);
@@ -210,6 +326,9 @@ static int start_subprocess(char *cmdexec, struct subprocess_info *info)
     if (cmdbuf != cmdexec)
         free(cmdbuf);
 
+    if (pid == -1)
+        return -1;
+
     if (register_subprocess(info->proc) == -1) {
         if (o.verbose)
             logdebug("Couldn't register subprocess with termination handler; not executing.\n");
@@ -246,7 +365,7 @@ static DWORD WINAPI subprocess_thread_func(void *data) {
 
     /* Three events we watch for: socket read, pipe read, and process end. */
     events[0] = (HANDLE) WSACreateEvent();
-    WSAEventSelect(info->fd, events[0], FD_READ | FD_CLOSE);
+    WSAEventSelect(info->fdn.fd, events[0], FD_READ | FD_CLOSE);
     events[1] = info->child_out_r;
     events[2] = info->proc;
 
@@ -264,22 +383,34 @@ static DWORD WINAPI subprocess_thread_func(void *data) {
         if (i == WAIT_OBJECT_0) {
             /* Read from socket, write to process. */
             char buffer[BUFSIZ];
+            int pending;
 
-            if (ReadFile((HANDLE) info->fd, buffer, sizeof(buffer), &n, NULL) == 0)
-                break;
-            if (n == -1 || n == 0)
-                break;
-            if (WriteFile(info->child_in_w, buffer, n, &nwritten, NULL) == 0)
-                break;
-            if (nwritten != n)
-                break;
-            ResetEvent(events[0]);
-        } else if (i == WAIT_OBJECT_0 + 1) {
-            /* Read from process, write to socket. */
-            if (GetOverlappedResult(info->child_out_r, &overlap, &n, FALSE)) {
-                if (WriteFile((HANDLE) info->fd, pipe_buffer, n, &nwritten, NULL) == 0)
+            do {
+                n = ncat_recv(&info->fdn, buffer, sizeof(buffer), &pending);
+                if (n <= 0)
+                    goto loop_end;
+                if (WriteFile(info->child_in_w, buffer, n, &nwritten, NULL) == 0)
                     break;
                 if (nwritten != n)
+                    goto loop_end;
+            } while (pending);
+            ResetEvent(events[0]);
+        } else if (i == WAIT_OBJECT_0 + 1) {
+            char *crlf = NULL, *wbuf;
+            /* Read from process, write to socket. */
+            if (GetOverlappedResult(info->child_out_r, &overlap, &n, FALSE)) {
+                int n_r;
+
+                wbuf = pipe_buffer;
+                n_r = n;
+                if (o.crlf) {
+                    if (fix_line_endings((char *) pipe_buffer, &n_r, &crlf))
+                        wbuf = crlf;
+                }
+                nwritten = ncat_send(&info->fdn, wbuf, n_r);
+                if (crlf != NULL)
+                    free(crlf);
+                if (nwritten != n_r)
                     break;
                 /* Queue another ansychronous read. */
                 ReadFile(info->child_out_r, pipe_buffer, sizeof(pipe_buffer), NULL, &overlap);
@@ -298,14 +429,21 @@ static DWORD WINAPI subprocess_thread_func(void *data) {
         }
     }
 
+loop_end:
+
     WSACloseEvent(events[0]);
 
     GetExitCodeProcess(info->proc, &ret);
     if (o.debug > 1)
         logdebug("Subprocess ended with exit code %d.\n", ret);
 
+    assert(WaitForSingleObject(pseudo_sigchld_mutex, INFINITE) == WAIT_OBJECT_0);
+    if (pseudo_sigchld_handler != NULL)
+        pseudo_sigchld_handler();
+    assert(ReleaseMutex(pseudo_sigchld_mutex) != 0);
+
     assert(unregister_subprocess(info->proc) != -1);
-    shutdown(info->fd, 2);
+    shutdown(info->fdn.fd, 2);
     subprocess_info_close(info);
     free(info);
 
@@ -357,6 +495,10 @@ static int register_subprocess(HANDLE proc)
         subprocesses_mutex = CreateMutex(NULL, FALSE, NULL);
         assert(subprocesses_mutex != NULL);
     }
+    if (pseudo_sigchld_mutex == NULL) {
+        pseudo_sigchld_mutex = CreateMutex(NULL, FALSE, NULL);
+        assert(pseudo_sigchld_mutex != NULL);
+    }
 
     assert(WaitForSingleObject(subprocesses_mutex, INFINITE) == WAIT_OBJECT_0);
 
