@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2009 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -17,7 +17,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -34,31 +34,39 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
- * insecure.org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
+ * distribution.  By sending these changes to Fyodor or one of the         *
+ * Insecure.Org development mailing lists, it is assumed that you are      *
+ * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because the *
+ * inability to relicense code has caused devastating problems for other   *
+ * Free Software projects (such as KDE and NASM).  We also occasionally    *
+ * relicense the code to third parties as discussed above.  If you wish to *
+ * specify special license conditions of your contributions, just say so   *
+ * when you send them.                                                     *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details (                               *
- * http://www.gnu.org/copyleft/gpl.html ).                                 *
+ * General Public License v2.0 for more details                            *
+ * (http://www.gnu.org/licenses/gpl-2.0.html).                             *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock.h 2866 2005-09-12 05:35:14Z fyodor $ */
+/* $Id: nsock.h 15192 2009-08-20 21:36:58Z david $ */
+
+/* Would you like to include pcap support in nsock?
+ * Pcap support code is currently unstable, so we give
+ * you a choice. In future this #define will be removed.*/
+#define HAVE_PCAP 1
 
 #ifndef NSOCK_H
 #define NSOCK_H
 
+/* Keep assert() defined for security reasons */
+#undef NDEBUG
+
 #include <sys/types.h>
 #ifndef WIN32
 #include <sys/socket.h>
@@ -71,10 +79,11 @@
 extern "C" {
 #endif
 
-// The read calls will generally return after reading at least this
-// much data so that the caller can process it and so that the
-// connection spewing data doesn't monopolize resources.  The caller
-// can always initiate another read request to ask for more.
+/* The read calls will generally return after reading at least this
+ * much data so that the caller can process it and so that the
+ * connection spewing data doesn't monopolize resources.  The caller
+ * can always initiate another read request to ask for more.
+ */
 #define NSOCK_READ_CHUNK_SIZE 0x8FFFF
 
 /*********************  TYPEDEFS ********************/
@@ -109,6 +118,8 @@ typedef unsigned long nsock_event_id;
 
 /* This is used to save SSL sessionids between SSL connections */
 typedef void *nsock_ssl_session;
+typedef void *nsock_ssl_ctx;
+typedef void *nsock_ssl;
 
 /********************   PROTOTYPES *******************/
 
@@ -123,9 +134,13 @@ typedef void *nsock_ssl_session;
    example you could do a series of 15 second runs, allowing you to do
    other stuff between them.  Or you could just schedule a timer to
    call you back every 15 seconds.*/
-enum nsock_loopstatus { NSOCK_LOOP_NOEVENTS = 2, NSOCK_LOOP_TIMEOUT, NSOCK_LOOP_ERROR };
+enum nsock_loopstatus { NSOCK_LOOP_NOEVENTS = 2, NSOCK_LOOP_TIMEOUT, NSOCK_LOOP_ERROR, NSOCK_LOOP_QUIT };
 enum nsock_loopstatus nsock_loop(nsock_pool nsp, int msec_timeout);
 
+/* Calling this function will cause nsock_loop to quit on its next iteration
+   with a return value of NSOCK_LOOP_QUIT. */
+void nsock_loop_quit(nsock_pool nsp);
+
 /* This next function returns the errno style error code -- which is only
    valid if the status is NSOCK_LOOP_ERROR was returned by nsock_loop() */
 int nsp_geterrorcode(nsock_pool nsp);
@@ -133,11 +148,14 @@ int nsp_geterrorcode(nsock_pool nsp);
 /* Every nsp has an ID that is unique across the program execution */
 unsigned long nsp_getid(nsock_pool nsp);
 
-// Note that nsi_get1_ssl_session will increment the usage count
-// of the SSL_SESSION, since nsock does a free when the nsi is
-// destroyed.  It's up to any calling function/etc to do a
-// SSL_SESSION_free() on it.  nsi_get0_ssl_session doesn't
-// increment, and is for informational purposes only.
+nsock_ssl nsi_getssl(nsock_iod nsockiod);
+
+/* Note that nsi_get1_ssl_session will increment the usage count
+ * of the SSL_SESSION, since nsock does a free when the nsi is
+ * destroyed.  It's up to any calling function/etc to do a
+ * SSL_SESSION_free() on it.  nsi_get0_ssl_session doesn't
+ * increment, and is for informational purposes only.
+ */
 nsock_ssl_session nsi_get1_ssl_session(nsock_iod nsockiod);
 nsock_ssl_session nsi_get0_ssl_session(nsock_iod nsockiod);
 
@@ -159,6 +177,16 @@ void *nsp_getud(nsock_pool nsp);
    (the time program execution starts would be a good candidate) */
 void nsp_settrace(nsock_pool nsp, int tracelevel, const struct timeval *basetime);
 
+/* Initializes an Nsock pool to create SSL connections. This sets an internal
+   SSL_CTX, which is like a template that sets options for all connections that
+   are made from it. Returns the SSL_CTX so you can set your own options. */
+nsock_ssl_ctx nsp_ssl_init(nsock_pool ms_pool);
+
+/* Initializes an Nsock pool to create SSL connections that emphasize speed over
+   security. Insecure ciphers are used when they are faster and no certificate
+   verification is done. Returns the SSL_CTX so you can set your own options. */
+nsock_ssl_ctx nsp_ssl_init_max_speed(nsock_pool ms_pool);
+
 /* And here is how you create an nsock_pool.  This allocates, initializes,
    and returns an nsock_pool event aggregator.  In the case of error,
    NULL will be returned.  If you do not wish to immediately associate
@@ -177,9 +205,15 @@ void nsp_delete(nsock_pool nsp);
 /* IF YOU ADD NEW NSE_TYPES YOU MUST INCREASE TYPE_CODE_NUM_BITS SO THAT
    IT IS ALWAYS log2(maximum_nse_type_value + 1) */
 #define TYPE_CODE_NUM_BITS 3
-enum nse_type { NSE_TYPE_CONNECT=0, NSE_TYPE_CONNECT_SSL=1, NSE_TYPE_READ=2, NSE_TYPE_WRITE=3, 
-		NSE_TYPE_TIMER=4 };  /* At some point I was considering a 
-				      NSE_TYPE_START and NSE_TYPE_CUSTOM */
+enum nse_type { 
+	NSE_TYPE_CONNECT=0,
+	NSE_TYPE_CONNECT_SSL=1,
+	NSE_TYPE_READ=2,
+	NSE_TYPE_WRITE=3, 
+	NSE_TYPE_TIMER=4,
+	NSE_TYPE_PCAP_READ=5,
+	NSE_TYPE_MAX=6,
+};  /* At some point I was considering a NSE_TYPE_START and NSE_TYPE_CUSTOM */
 
 /* Find the type of an event that spawned a callback */
 enum nse_type nse_type(nsock_event nse);
@@ -254,9 +288,9 @@ nsock_iod nsi_new(nsock_pool nsockp, void *userdata);
 /* This version allows you to associate an existing sd with the msi
    so that you can read/write it using the nsock infrastructure.  For example,
    you may want to watch for data from STDIN_FILENO at the same time as you
-   read/wrtie various sockets. Ths sd is dup()ed, so you may close or
-   otherwise manipulate your copy.  The duped copy will be destroyed when the
-   nsi is destroyed 
+   read/write various sockets.  STDIN_FILENO is a special case, however. Any
+   other sd is dup()ed, so you may close or otherwise manipulate your copy.
+   The duped copy will be destroyed when the nsi is destroyed
 */
 nsock_iod nsi_new2(nsock_pool nsockp, int sd, void *userdata);
 
@@ -296,9 +330,29 @@ int nsi_getsd(nsock_iod nsiod);
    ids for a given nspool (unless you blow through billions of them). */
 unsigned long nsi_id(nsock_iod nsockiod);
 
+/*Returns Packets received in bytes   */
+unsigned long nsi_get_read_count(nsock_iod nsockiod);
+
+/*Returns Packets sent in bytes   */
+unsigned long nsi_get_write_count(nsock_iod nsockiod);
+
   /* Returns 1 if an NSI is communicating via SSL, 0 otherwise */
 int nsi_checkssl(nsock_iod nsockiod);
 
+/* Returns the remote peer port (or -1 if unavailable).  Note the
+   return value is a whole int so that -1 can be distinguished from
+   65535.  Port is returned in host byte order. */
+int nsi_peerport(nsock_iod nsiod);
+
+/* Sets the local address to bind to before connect() */
+int nsi_set_localaddr(nsock_iod nsi, struct sockaddr_storage *ss, size_t sslen);
+
+/* Sets IPv4 options to apply before connect().  It makes a copy of the 
+ * options, so you can free() yours if necessary.  This copy is freed
+ * when the iod is destroyed
+ */
+int nsi_set_ipoptions(nsock_iod nsi, void *ipopts, size_t ipoptslen);
+
 /* Returns that host/port/protocol information for the last
    communication (or comm. attempt) this nsi has been involved with.
    By "involved" with I mean interactions like establishing (or trying
@@ -364,6 +418,17 @@ nsock_event_id nsock_connect_tcp(nsock_pool nsp, nsock_iod nsiod,
 				 void *userdata, struct sockaddr *ss,
 				 size_t sslen, unsigned short port);
 
+/* Request an SCTP association to another system (by IP address).  The
+   in_addr is normal network byte order, but the port number should be
+   given in HOST BYTE ORDER.  ss should be a sockaddr_storage,
+   sockaddr_in6, or sockaddr_in as appropriate (just like what you
+   would pass to connect).  sslen should be the sizeof the structure
+   you are passing in. */
+nsock_event_id nsock_connect_sctp(nsock_pool nsp, nsock_iod nsiod, 
+				 nsock_ev_handler handler, int timeout_msecs, 
+				 void *userdata, struct sockaddr *ss,
+				 size_t sslen, unsigned short port);
+
 /* Request a UDP "connection" to another system (by IP address).  The
    in_addr is normal network byte order, but the port number should be
    given in HOST BYTE ORDER.  Since this is UDP, no packets are
@@ -385,10 +450,10 @@ nsock_event_id nsock_connect_udp(nsock_pool nsp, nsock_iod nsiod,
 				 struct sockaddr *ss, size_t sslen,
 				 unsigned short port);
 
-/* Request an SSL over TCP connection to another system (by IP
+/* Request an SSL over TCP/SCTP connection to another system (by IP
    address).  The in_addr is normal network byte order, but the port
    number should be given in HOST BYTE ORDER.  This function will call
-   back only after it has made the TCP connection AND done the initial
+   back only after it has made the connection AND done the initial
    SSL negotiation.  From that point on, you use the normal read/write
    calls and decryption will happen transparently. ss should be a
    sockaddr_storage, sockaddr_in6, or sockaddr_in as appropriate (just
@@ -397,9 +462,17 @@ nsock_event_id nsock_connect_udp(nsock_pool nsp, nsock_iod nsiod,
 nsock_event_id nsock_connect_ssl(nsock_pool nsp, nsock_iod nsiod, 
 				 nsock_ev_handler handler, int timeout_msecs, 
 				 void *userdata, struct sockaddr *ss, 
-				 size_t sslen, unsigned short port,
+				 size_t sslen, int proto, unsigned short port,
 				 nsock_ssl_session ssl_session);
 
+/* Request ssl connection over already established TCP/SCTP connection.
+   nsiod must be socket that is already connected to target
+   using nsock_connect_tcp or nsock_connect_sctp.
+   All parameters have the same meaning as in 'nsock_connect_ssl' */
+nsock_event_id nsock_reconnect_ssl(nsock_pool nsp, nsock_iod nsiod,
+				   nsock_ev_handler handler, int timeout_msecs,
+                                   void *userdata, nsock_ssl_session ssl_session);
+
 /* Read up to nlines lines (terminated with \n, which of course
    inclues \r\n), or until EOF, or until the timeout, whichever comes
    first.  Note that NSE_STATUS_SUCCESS will be returned in the case
@@ -452,6 +525,54 @@ int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify );
    before returning */
 const struct timeval *nsock_gettimeofday();
 
+
+#ifdef HAVE_PCAP
+/*
+ * Open pcap device and connect it to nsp. Other parameters have the
+ * same meaning as for pcap_open_live in pcap(3).
+ * 	device   : pcap-style device name
+ * 	snaplen  : size of packet to be copied to handler
+ * 	promisc  : whether to open device in promiscous mode
+ * 	bpf_fmt	 : berkeley filter 
+ * return value: NULL if everything was okay, or error string if error occurred
+ * [sorry Fyodor for breaking the API, but it's just simpler]
+ * */
+char *nsock_pcap_open(nsock_pool nsp, nsock_iod nsiod, 
+		const char *pcap_device, int snaplen, int promisc,
+		const char *bpf_fmt, ...);
+
+/*
+ * Requests exacly one packet to be captured.from pcap. 
+ * See nsock_read() for parameters description.
+ * */
+nsock_event_id nsock_pcap_read_packet(nsock_pool nsp, nsock_iod nsiod, 
+			 nsock_ev_handler handler, int timeout_msecs,
+			 void *userdata);
+
+/*
+ * Gets packet data. This should be called after succesfull receiving of packet
+ * to get packet.  If you're not interested in some values, just pass NULL
+ * instead of valid pointer.
+ * l3_data is just after l2_data in buffer. Feel free to treat l2_data as one
+ * buffer with size of (l2_len + l3_len). 
+ * Ts time is fixed for systems that don't support proper timing, like Windows.
+ * So TS is pointing to time when packet was received or to the time _after_.
+ * As a result you'll get longer times than you should, but it's safer to
+ * think that host is a bit further. 
+ * */
+void nse_readpcap(nsock_event nsee,
+	const unsigned char **l2_data, size_t *l2_len,
+	const unsigned char **l3_data, size_t *l3_len,
+	size_t *packet_len, struct timeval *ts); 
+
+/* Well. Just pcap-style datalink. Like DLT_EN10MB or DLT_SLIP. Check in pcap(3) manpage. */
+int nsi_pcap_linktype(nsock_iod nsiod);
+
+/* Is this nsiod a pcap descriptor? */
+int nsi_is_pcap(nsock_iod nsiod);
+
+#endif /* HAVE_PCAP */
+
 #ifdef __cplusplus
 } /* End of 'extern "C"' */
 #endif
@@ -464,3 +585,5 @@ const struct timeval *nsock_gettimeofday();
 
 
 
+
+
